<?php

/**
 *	       __          __                __            
 *	  ____/ /__ _   __/ /_  ____  ____  / /__ _________
 *	 / __  / _ \ | / / __ \/ __ \/ __ \/ //_// ___/_  /
 *	/ /_/ /  __/ |/ / /_/ / /_/ / /_/ / ,< _/ /__  / /_
 *	\__,_/\___/|___/_.___/\____/\____/_/|_(_)___/ /___/
 *                                                   
 *                                                           
 *      TUTORILY  <>  DISKUZE  <>  KOMUNITA  <>  SOFTWARE
 * 
 *	Tento zdrojov kd je soust tutoril na programtorsk 
 *	sociln sti WWW.DEVBOOK.CZ	
 *	
 *	Kd mete upravovat jak chcete, jen zmite odkaz 
 *	na www.devbook.cz :-) 
 * 
 *  Jednoduch databzov wrapper nad PDO
 */
class Db
{
	/**
	 * @var PDO Databzov spojen
	 */
	private static $connection;

	/**
	 * @var array Vchoz nastaven ovladae
	 */
	private static $options = array(
		PDO::ATTR_ERRMODE => PDO::ERRMODE_WARNING,
		PDO::MYSQL_ATTR_INIT_COMMAND => "SET NAMES utf8",
		PDO::ATTR_EMULATE_PREPARES => false,
	);

	/**
	 * Pipoj se k databzi pomoc danch daj
	 * @param string $host Nzev hostitele
	 * @param string $database Nzev databze
	 * @param string $user Uivatelsk jmno
	 * @param string $password Heslo
	 */
	public static function connect($host, $database, $user, $password)
	{
		if (!isset(self::$connection)) {
			$dsn = "mysql:host=$host;dbname=$database";
			self::$connection = new PDO($dsn, $user, $password, self::$options);
		}
	}

	/**
	 * Spust dotaz a vrt PDO statement
	 * @param array $params Pole, kde je prvnm prvkem dotaz a dalmi jsou parametry
	 * @return \PDOStatement PDO statement
	 */
	private static function executeStatement($params)
	{
		$query = array_shift($params);
		$statement = self::$connection->prepare($query);
		$statement->execute($params);
		return $statement;
	}

	/**
	 * Spust dotaz a vrt poet ovlivnnch dk. Dle se ped libovoln poet dalch parametr.
	 * @param string $query Dotaz
	 * @return int Poet ovlivnnch dk
	 */
	public static function query($query) {
		$statement = self::executeStatement(func_get_args());
		return $statement->rowCount();
	}

	/**
	 * Spust dotaz a vrt z nj prvn sloupec prvnho dku. Dle se ped libovoln poet dalch parametr.
	 * @param string $query Dotaz
	 * @return mixed Hodnota prvnho sloupce z prvnho dku
	 */
	public static function querySingle($query) {
		$statement = self::executeStatement(func_get_args());
		$data = $statement->fetch();
		return $data[0];
	}

	/**
	 * Spust dotaz a vrt z nj prvn dek. Dle se ped libovoln poet dalch parametr.
	 * @param string $query Dotaz
	 * @return mixed Pole vsledk nebo false pi nespchu
	 */
	public static function queryOne($query) {
		$statement = self::executeStatement(func_get_args());
		return $statement->fetch(PDO::FETCH_ASSOC);
	}

	/**
	 * Spust dotaz a vrt vechny jeho dky jako pole asociativnch pol. Dle se ped libovoln poet dalch parametr.
	 * @param string $query Dotaz
	 * @return mixed Pole dk enbo false pi nespchu
	 */
	public static function queryAll($query) {
		$statement = self::executeStatement(func_get_args());
		return $statement->fetchAll(PDO::FETCH_ASSOC);
	}

	/**
	 * Umouje snadn vloen zznamu do databze pomoc asociativnho pole
	 * @param string $table Nzev tabulky
	 * @param array $data Asociativn pole, kde jsou kle sloupce a hodnoty hodnoty
	 * @return int Poet ovlivnnch dk
	 */
	public static function insert($table, $data) {
		$keys = array_keys($data);
		self::checkIdentifiers(array($table) + $keys);
		$query = "
			INSERT INTO `$table` (`" . implode('`, `', $keys) . "`)
			VALUES (" . str_repeat('?,', count($data) - 1) . "?)
		";
		$params = array_merge(array($query), array_values($data));
		$statement = self::executeStatement($params);
		return $statement->rowCount();
	}

	/**
	 * Umouje snadnou modifikaci zznamu v databzi pomoc asociativnho pole
	 * @param string $table Nzev tabulky
	 * @param array $data Asociativn pole, kde jsou kle sloupce a hodnoty hodnoty
	 * @param string $condition etzec s SQL podmnkou (WHERE)
	 * @return mixed
	 */
	public static function update($table, $data, $condition) {
		$keys = array_keys($data);
		self::checkIdentifiers(array($table) + $keys);
		$query = "
			UPDATE `$table` SET `".
			implode('` = ?, `', array_keys($data)) . "` = ?
			$condition
		";
		$params = array_merge(array($query), array_values($data), array_slice(func_get_args(), 3));
		$statement = self::executeStatement($params);
		return $statement->rowCount();
	}

	/**
	 * Vrt posledn ID poslednho zznamu vloenho pomoc INSERT
	 * @return mixed Id poslednho zznamu
	 */
	public static function getLastId()
	{
		return self::$connection->lastInsertId();
	}

	/**
	 * Oet string proti SQL injekci
	 * @param string $string etzec
	 * @return mixed Oeten etzec
	 */
	public static function quote($string)
	{
		return self::$connection->quote($string);
	}

	/**
	 * Zkontroluje, zda identifiktory odpovdaj formtu identifiktor
	 * @param array $identifiers Pole identifiktor
	 * @throws \Exception
	 */
	private static function checkIdentifiers($identifiers)
	{
		foreach ($identifiers as $identifier)
		{
			if (!preg_match('/^[a-zA-Z0-9\_\-]+$/u', $identifier))
				throw new Exception('Dangerous identifier in SQL query');
		}
	}
}